classdef RecommendedParameters
    %RecommendedParameters is a class containing recommended parameters for
    %   different preprocessing steps. This is different from the
    %   DefaultParameters.m in the sense that there is a recommended value
    %   even for preprocessing steps that are not used by default: These
    %   structures must have all the possible fields for each parameter
    %   (with the exception of PrepParams), so that they are used later in
    %   the GUI or other places where no default parameter is given but
    %   another recommendation is needed. 
    %
    %   Please do not change anything in this file unless you are sure what
    %   you are doing.
    %
    % Copyright (C) 2017  Amirreza Bahreini, methlabuzh@gmail.com
    % 
    % This program is free software: you can redistribute it and/or modify
    % it under the terms of the GNU General Public License as published by
    % the Free Software Foundation, either version 3 of the License, or
    % (at your option) any later version.
    % 
    % This program is distributed in the hope that it will be useful,
    % but WITHOUT ANY WARRANTY; without even the implied warranty of
    % MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    % GNU General Public License for more details.
    % 
    % You should have received a copy of the GNU General Public License
    % along with this program.  If not, see <http://www.gnu.org/licenses/>.
    properties(Constant)          
       FilterParams = struct('notch',    struct('freq', 50),...
                             'high',     struct('freq', 0.1,...
                                                 'order', []),... % Default
                             'low',      struct('freq', 30,...
                                                 'order', []),... % Default
                             'zapline',   struct('freq', 50,...
                                                'ncomps', 3))
        CRDParams = struct('ChannelCriterion',     0.85,...
                           'LineNoiseCriterion',   4,...
                           'BurstCriterion',       5,...
                           'WindowCriterion',      0.25, ...
                           'Highpass',             [0.25 0.75]);
        
        PrepParams = struct();                           % Default by prep
        
        HighvarParams = struct('sd', 25, ...
                               'cutoff', 100, ...
                               'rejRatio', 0.5);
        
        MinvarParams = struct('sd', 1);
        
        InterpolationParams = struct('method', 'spherical');
        
        RPCAParams = struct('lambda', [], ...  % Default lambda by alm_rpca
                            'tol', 1e-7, ...
                            'maxIter', 1000);
        
        MARAParams = struct('chanlocMap', containers.Map, ...
                            'largeMap',   0, ...
                            'high',       struct('freq', 2.0,...
                                                 'order', []), ...
                                                 'keep_comps', 0);
                                              
        
        ICLabelParams = struct('brainTher', [], ...
                               'muscleTher', 0.8, ...
                               'eyeTher', 0.8, ...
                               'heartTher', 0.8, ...
                               'lineNoiseTher', 0.8, ...
                               'channelNoiseTher', 0.8, ...
                               'otherTher', 0.8, ...
                               'includeSelected', 0, ...
                               'high',     struct('freq', 2.0,...
                                                  'order', []));
        
        EOGRegressionParams = struct();
        
        DetrendingParams = struct();
                        
        ChannelReductionParams = struct('tobeExcludedChans', []);
                                      
                                      
        EEGSystem = struct('name', 'Others',...
                           'sys10_20', 0, ...
                           'locFile', '', ...
                           'refChan', struct('idx', []), ...
                           'fileLocType', '',...
                           'eogChans', [], ...
                           'powerLineFreq', []);
         
        Settings = struct('trackAllSteps', 0, ...
                          'pathToSteps', '/allSteps.mat');
    end
end