function electrodes_to_tsv(EEG)

% From an EEG variable (i.e. EEG=pop_loadset(*.set), export the EEG.channel
% locations as tsv file and initiate the json file following the BIDS 
% specification
%
% FORMAT electrodes_to_tsv(EEG)
%
% Author: Cyril Pernet - LIMO Team, University of Edinurgh

% electrode.tsv

N = max(size(EEG.chanlocs));
ename = cell(N); 
x = zeros(N);
y = zeros(N);
z = zeros(N);
type = zeros(N);
    
for electrode = 1:N
    ename{electrode} = EEG.chanlocs(electrode).labels; 
    if ~isempty(EEG.chanlocs(electrode).X) % modified by amirreza
        x(electrode)     = EEG.chanlocs(electrode).X;
        y(electrode)     = EEG.chanlocs(electrode).Y;
        z(electrode)     = EEG.chanlocs(electrode).Z;
        type{electrode}  = EEG.chanlocs(electrode).type;
    end
end

t = table(ename',x',y',z',type','VariableNames',{'name','x','y','z','type'});
electrodes_tsv_name = [EEG.filepath(1:end-4) 'electrodes.tsv'];
writetable(t, electrodes_tsv_name,'FileType','text','Delimiter','\t');

% coordsystem.json
EEGCoordinateSystem = 'RAS';
EEGCoordinateUnits  = 'mm';
json = struct('EEGCoordinateSystem','RAS', ...
        'EEGCoordinateUnits','mm');
text = jsonencode(json);
fid = fopen([EEG.filepath(1:end-4) 'electrodes.json'], 'w');
fwrite(fid, text, 'char');
fclose(fid);


